using System;

namespace QTPlugin
{
	/// <summary>
	/// Specifies the informations about plugin.
	/// QTTabBar use this in option window.
	/// </summary>
	[AttributeUsage( AttributeTargets.Class )]
	public sealed class PluginAttribute : Attribute
	{
		/// <summary>
		/// Can be a nickname.
		/// </summary>
		public string Author;

		/// <summary>
		/// Name of the plugin. 
		/// </summary>
		public string Name;

		/// <summary>
		/// Version string like "1.0.0.0"
		/// </summary>
		public string Version;

		/// <summary>
		/// Description of the plugin. shown in Option -> Plugin
		/// </summary>
		public string Description;

		/// <summary>
		/// Interactive, Background, etc.
		/// </summary>
		public PluginType PluginType;


		/// <summary>
		/// </summary>
		/// <param name="pluginType">Plugin type. Interactive or Background.</param>
		public PluginAttribute( PluginType pluginType )
		{
			this.PluginType = pluginType;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="pluginType"></param>
		/// <param name="name"></param>
		/// <param name="author"></param>
		/// <param name="version"></param>
		/// <param name="description"></param>
		public PluginAttribute( PluginType pluginType, string name, string author, string version, string description )
		{
			this.PluginType = pluginType;
			this.Name = name;
			this.Author = author;
			this.Version = version;
			this.Description = description;
		}

		/// <summary>
		/// New in 1.0.1.0
		/// Create strings on execution time.
		/// 
		/// (recommended)
		/// </summary>
		/// <param name="pluginType">PluginType enum.</param>
		/// <param name="tProvider">Type of the class which implements LocalizedStringProvider.</param>
		/// <param name="iKey">
		/// Plugin index in the plugin assembly. 
		/// This is passed to LocalizedStringProvider.SetKey() method in this constructor.
		/// </param>
		public PluginAttribute( PluginType pluginType, Type tProvider, int iKey )
		{
			this.PluginType = pluginType;

			if( tProvider.IsSubclassOf( typeof( LocalizedStringProvider ) ) )
			{
				try
				{
					LocalizedStringProvider provider = (LocalizedStringProvider)Activator.CreateInstance( tProvider );
					provider.SetKey( iKey );
					this.Author = provider.Author;
					this.Name = provider.Name;
					this.Description = provider.Description;
				}
				catch( MissingMethodException )
				{
					this.Author = String.Empty;
					this.Name = "Name missing";
					this.Description = "Default constuctor of LocalizedStringProvider is missing. \nContact the author of this plugin.";
				}
			}
			else
			{
				this.Author = String.Empty;
				this.Name = "Name missing";
				this.Description = "The type is not subclass of LocalizedStringProvider. \nContact the author of this plugin.";
			}
		}

		/// <summary>
		/// New in 1.0.1.0
		/// Create strings on execution time.
		/// </summary>
		/// <param name="pluginType"></param>
		/// <param name="tProvider"></param>
		public PluginAttribute( PluginType pluginType, Type tProvider )
			: this( pluginType, tProvider, -1 )
		{
		}

	}


	/// <summary>
	/// New in 1.0.1.0
	/// Implement this abstract class and give the Type object of it to constructor of PluginAttribute.
	/// 
	/// You can provide localized strings on execution time using CultureInfo.CurrentCulture.
	/// Implementing class needs to have default constructor (a constructor without arguments).
	/// </summary>
	public abstract class LocalizedStringProvider
	{
		/// <summary>
		/// Use this method to know which plugin type is being instancialized in Assembly that contains multiple plugin classes.
		/// If your assembly has just one plugin class, ignore this.
		/// 
		/// It's guaranteed that this call precedes getting Name, Author and Description.
		/// See PluginAttribute..ctor( PluginType, Type, int ).
		/// </summary>
		/// <param name="iKey"></param>
		public abstract void SetKey( int iKey );

		/// <summary>
		/// Return display name.
		/// </summary>
		public abstract string Name
		{
			get;
		}

		/// <summary>
		/// 
		/// </summary>
		public abstract string Author
		{
			get;
		}

		/// <summary>
		/// 
		/// </summary>
		public abstract string Description
		{
			get;
		}
	}
}
