using System;
using System.Collections.Generic;
using System.Drawing;
using System.Globalization;
using System.IO;
using System.Text;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Microsoft.Win32;

using QTPlugin;
using QTPlugin.Interop;

namespace QuizoPlugins
{
	[Plugin( PluginType.Background, typeof(Localizer), Version = "1.1.0.0" )]
	public class CreateNewItemButton : IBarButton
	{
		private IPluginServer pluginServer;
		private IShellBrowser shellBrowser;

		internal static string REGNAME = "QTCreateNewItems";


		[DllImport( "shell32.dll", CharSet = CharSet.Auto )]
		private static extern IntPtr ILCreateFromPath( [MarshalAs( UnmanagedType.LPTStr )] string pszPath );

		[DllImport( "shell32.dll" )]
		private static extern IntPtr ILFindLastID( IntPtr pidl );

		[DllImport( "user32.dll" )]
		private static extern bool GetWindowRect( IntPtr hWnd, out RECT pRect );



		public static void Uninstall()
		{
			using( RegistryKey rkPlugin = Registry.CurrentUser.OpenSubKey( CONSTANTS.REGISTRY_PLUGINSETTINGS, true ) )
			{
				if( rkPlugin != null )
				{
					rkPlugin.DeleteSubKey( REGNAME, false );
				}
			} 
		}


		#region IPluginClient members

		public void Open( IPluginServer pluginServer, IShellBrowser shellBrowser )
		{
			this.pluginServer = pluginServer;
			this.shellBrowser = shellBrowser;
		}

		public bool QueryShortcutKeys( out string[] actions )
		{
			actions = new string[] { StringResources.ShortcutKeys[0], StringResources.ShortcutKeys[1], StringResources.ShortcutKeys[2], StringResources.ShortcutKeys[3] };
			return true;
		}

		public void Close( EndCode endCode )
		{
		}

		public bool HasOption
		{
			get
			{
				return false;
			}
		}

		public void OnMenuItemClick( MenuType menuType, string menuText, ITab tab )
		{
		}

		public void OnOption()
		{
		}

		public void OnShortcutKeyPressed( int index )
		{
			switch( index )
			{
				case 0:
				case 1:
					this.CreateAFile( index == 0 );
					break;

				case 2:
					this.OpenFileCreateDialog();
					break;

				case 3:
					this.OpenNewFolderInTab( false );
					break;
			}
		}

		#endregion


		#region IBarButton members

		public Image GetImage( bool fLarge )
		{
			return fLarge ? Resource.CreateNewItemButton_large : Resource.CreateNewItemButton_small;
		}

		public void InitializeItem()
		{
		}

		public void OnButtonClick()
		{
			this.OpenFileCreateDialog();
		}

		public bool ShowTextLabel
		{
			get
			{
				return false;
			}
		}

		public string Text
		{
			get
			{
				return StringResources.Name;
			}
		}

		#endregion


		private void CreateAFile( bool fDir )
		{
			const uint SVSI_SELECT = 0x00000001;
			const uint SVSI_EDIT = 0x00000003;
			const uint SVSI_DESELECTOTHERS = 0x00000004;
			const uint SVSI_ENSUREVISIBLE = 0x00000008;

			// Create folder or file
			IShellView shellView = null;
			IntPtr pIDL = IntPtr.Zero;

			try
			{
				string path = this.pluginServer.SelectedTab.Address.Path;

				if( String.IsNullOrEmpty( path ) || !Directory.Exists( path ) )
				{
					System.Media.SystemSounds.Hand.Play();
					return;
				}

				// make new name
				int i = 2;
				string name = fDir ? StringResources.NewFileNames[0] : StringResources.NewFileNames[1];
				string ext = fDir ? String.Empty : ".txt";
				string pathNew = path + "\\" + name + ext;

				while( ( fDir && Directory.Exists( pathNew ) ) || ( !fDir && File.Exists( pathNew ) ) )
				{
					pathNew = path + "\\" + name + " (" + i + ")" + ext;
					i++;
				}

				if( fDir )
					Directory.CreateDirectory( pathNew );
				else
					using( File.Create( pathNew ) )
					{
					}


				// Select and put into rename mode.
				if( 0 == this.shellBrowser.QueryActiveShellView( out shellView ) )
				{
					shellView.Refresh();

					pIDL = ILCreateFromPath( pathNew );
					if( pIDL != IntPtr.Zero )
					{
						IntPtr pIDLRltv = ILFindLastID( pIDL );
						if( pIDLRltv != IntPtr.Zero )
						{
							shellView.SelectItem( pIDLRltv, SVSI_SELECT | SVSI_DESELECTOTHERS | SVSI_ENSUREVISIBLE | SVSI_EDIT );
							return;
						}
					}
				}
			}
			catch
			{
			}
			finally
			{
				if( shellView != null )
					Marshal.ReleaseComObject( shellView );

				if( pIDL != IntPtr.Zero )
					Marshal.FreeCoTaskMem( pIDL );
			}

			System.Media.SystemSounds.Hand.Play();
		}

		private void OpenFileCreateDialog()
		{
			ITab tab = this.pluginServer.SelectedTab;
			if( tab != null )
			{
				string pathCurrent = tab.Address.Path;
				if( Directory.Exists( pathCurrent ) )
				{
					using( FormCreateItems fci = new FormCreateItems( pathCurrent, this.pluginServer ) )
					{
						this.pluginServer.ExecuteCommand( Commands.SetModalState, true );
						try
						{
							fci.ShowDialog();
						}
						finally
						{
							this.pluginServer.ExecuteCommand( Commands.SetModalState, false );
						}
					}
				}
				else
					System.Media.SystemSounds.Beep.Play();
			}
		}

		private void OpenNewFolderInTab( bool fNewWindow )
		{
			try
			{
				ITab tab = this.pluginServer.SelectedTab;
				if( tab != null )
				{
					string pathCurrent = tab.Address.Path;
					if( Directory.Exists( pathCurrent ) )
					{
						string pathNewFolder = String.Empty;

						this.pluginServer.ExecuteCommand( Commands.SetModalState, true );
						try
						{
							using( EnterNewNameForm ennf = new EnterNewNameForm() )
							{
								RECT rct;
								GetWindowRect( this.pluginServer.ExplorerHandle, out rct );

								ennf.Location = new Point( rct.left + ( rct.Width - ennf.Width ) / 2, rct.top + ( rct.Height - ennf.Height ) / 2 );

								if( DialogResult.OK != ennf.ShowDialog() || ennf.NewName.Length == 0 )
								{
									return;
								}

								pathNewFolder = pathCurrent + "\\" + CreateNewItemButton.SanitizeFileNameString( ennf.NewName );
							}
						}
						finally
						{
							this.pluginServer.ExecuteCommand( Commands.SetModalState, false );
						}

						if( Directory.Exists( pathNewFolder ) )
						{
							System.Media.SystemSounds.Hand.Play();
						}
						else
						{
							Directory.CreateDirectory( pathNewFolder );
							this.pluginServer.CreateTab( new Address( pathNewFolder ), -1, false, true );
						}
					}
				}
			}
			catch
			{
			}
		}

		private static string SanitizeFileNameString( string path )
		{
			if( path == null )
				return null;

			path = path.Trim();

			StringBuilder sb = new StringBuilder( path.Length );
			for( int i = 0; i < path.Length; i++ )
			{
				if( IsValidFileNameChar( path[i] ) )
				{
					sb.Append( path[i] );
				}
			}
			return sb.ToString();
		}

		internal static bool IsValidFileNameChar( char ch )
		{
			// ", <, >, |, \, /, :, *, ? 

			return ( ch != '"' ) && ( ch != '<' ) && ( ch != '>' ) && ( ch != '|' ) && ( ch != '\\' ) && ( ch != '/' ) && ( ch != ':' ) && ( ch != '*' ) && ( ch != '?' );	//&& ( ch > 0x1f )
		}

	}

	static class StringResources
	{
		public static string[] NewFileNames, Tooltips, Messages, ShortcutKeys;
		public static string Name;

		static StringResources()
		{
			char[] SEPCHAR = new char[] { ';' };

			if( System.Globalization.CultureInfo.CurrentCulture.Parent.Name == "ja" )
			{
				NewFileNames = Resource.NewFileNames_ja.Split( SEPCHAR );
				Messages = Resource.Messages_ja.Split( SEPCHAR );
				ShortcutKeys = Resource.Shortcutkeys_ja.Split( SEPCHAR );
				Tooltips = Resource.Tooltips_ja.Split( SEPCHAR );

				Name = Resource.Name_ja;
			}
			else
			{
				NewFileNames = Resource.NewFileNames.Split( SEPCHAR );
				Messages = Resource.Messages.Split( SEPCHAR );
				ShortcutKeys = Resource.Shortcutkeys.Split( SEPCHAR );
				Tooltips = Resource.Tooltips.Split( SEPCHAR );

				Name = Resource.Name;
			}
		}
	}

	sealed class Localizer: LocalizedStringProvider
	{
		private string parent;

		public Localizer()
		{
			this.parent = CultureInfo.CurrentCulture.Parent.Name;
		}

		public override string Author
		{
			get
			{
				return "Quizo";
			}
		}

		public override string Description
		{
			get
			{
				if( this.parent == "ja" )
				{
					return "t@CtH_쐬郆[eBeBłB\nAԃt@C쐬At@CEtH_쐬ɃL[{[hV[gJbg񋟂܂B";
				}
				else
				{
					return "Utility tool to create files and folder adding sequential numbers.";
				}
			}
		}

		public override string Name
		{
			get
			{
				if( this.parent == "ja" )
				{
					return "AԃACe쐬";
				}
				else
				{
					return "Create file button";
				}
			}
		}

		public override void SetKey( int iKey )
		{			
		}
	}
}
