using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Globalization;
using System.IO;
using System.Windows.Forms;
using Microsoft.Win32;

using QTPlugin;
using QTPlugin.Interop;

namespace QuizoPlugins
{
	[Plugin( PluginType.BackgroundMultiple, typeof(Localizer), Version = "1.1.0.0" )]
	public sealed class LauncherButton : IBarMultipleCustomItems
	{
		private IPluginServer pluginServer;
		private IShellBrowser shellBrowser;

		private List<LauncherData> lstData = new List<LauncherData>();
		private List<ToolStripButtonEx> lstBtns = new List<ToolStripButtonEx>();

		private bool fCurrentLarge;
		private Localizer localizer = new Localizer();

		private static string PREFIX_GROUP = "group:";
		private static string PREFIX_TAB = "tab:";


		#region IPluginClient members

		public void Open( IPluginServer pluginServer, IShellBrowser shellBrowser )
		{
			this.pluginServer = pluginServer;
			this.shellBrowser = shellBrowser;
		}

		public bool QueryShortcutKeys( out string[] actions )
		{
			int c = this.ReadSettings( false );
			if( c > 0 )
			{
				List<string> lstSK = new List<string>();
				for( int i = 1; i < c + 1; i++ )
				{
					lstSK.Add( String.Format( this.localizer.ShortcutKeyTemplate, i ) );
				}
				actions = lstSK.ToArray();
				return true;
			}
			else
			{
				actions = null;
				return false;
			}
		}

		public void Close( EndCode endCode )
		{
			if( endCode != EndCode.Hidden )
			{
				this.Initialize( null );
				this.ReleaseLauncherData();
			}
		}

		public bool HasOption
		{
			get
			{
				return true;
			}
		}

		public void OnMenuItemClick( MenuType menuType, string menuText, ITab tab )
		{
		}

		public void OnOption()
		{
			this.pluginServer.ExecuteCommand( Commands.SetModalState, true );
			try
			{
				using( OptionForm of = new OptionForm( this.lstData ) )
				{
					if( DialogResult.OK == of.ShowDialog() )
					{
						List<LauncherData> lstOldData = this.lstData;
						this.lstData = of.GetData();
						this.RefreshButtons();

						foreach( LauncherData ldOld in lstOldData )
						{
							ldOld.Dispose();
						}
						this.SaveSettings();
					}
				}
			}
			finally
			{
				this.pluginServer.ExecuteCommand( Commands.SetModalState, false );
			}
		}

		public void OnShortcutKeyPressed( int index )
		{
			if( -1 < index && index < this.lstData.Count )
			{
				this.Execute( this.lstData[index] );
			}
		}

		#endregion


		#region IBarMultipleCustomItems members

		public void Initialize( int[] order )
		{
			foreach( ToolStripButton tsb in this.lstBtns )
			{
				tsb.Dispose();
			}
			this.lstBtns.Clear();

			this.ReadSettings( true );


			if( order != null )
			{
				List<LauncherData> lstData2 = new List<LauncherData>();
				for( int i = 0; i < order.Length; i++ )
				{
					if( -1 < order[i] && order[i] < this.lstData.Count )
					{
						lstData2.Add( this.lstData[order[i]] );
					}
				}
				foreach( LauncherData ld in this.lstData )
				{
					if( !lstData2.Contains( ld ) )
						lstData2.Add( ld );
				}
				this.lstData = lstData2;
				this.SaveSettings();
			}
		}

		public ToolStripItem CreateItem( bool fLarge, DisplayStyle displayStyle, int index )
		{
			this.fCurrentLarge = fLarge;

			if( -1 < index && index < this.lstData.Count )
			{
				this.lstData[index].Instancialized = true;

				ToolStripButtonEx tsb = this.CreateButton( this.lstData[index], fLarge );
				this.lstBtns.Add( tsb );
				return tsb;
			}
			else
			{
				ToolStripButtonEx tsb = this.CreateDefBtn();
				this.lstBtns.Add( tsb );
				return tsb;
			}
		}

		public int Count
		{
			get
			{
				return -1;
			}
		}

		public Image GetImage( bool fLarge, int index )
		{
			if( this.lstData.Count > index )
			{
				if( fLarge )
				{
					if( this.lstData[index].ImageLarge != null )
						return this.lstData[index].ImageLarge;
				}
				else
				{
					if( this.lstData[index].ImageSmall != null )
						return this.lstData[index].ImageSmall;					
				}
			}
			return fLarge ? Resource.LauncherButton_large : Resource.LauncherButton_small;
		}

		public string GetName( int index )
		{
			if( this.lstData.Count > index )
			{
				return this.lstData[index].Name;
			}
			return this.localizer.Name;
		}

		#endregion


		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		private ToolStripButtonEx CreateDefBtn()
		{
			ToolStripButtonEx tsb = new ToolStripButtonEx();
			tsb.Image = this.fCurrentLarge ? Resource.LauncherButton_large : Resource.LauncherButton_small;
			tsb.ImageScaling = ToolStripItemImageScaling.None;
			tsb.AutoSize = false;
			tsb.Size = this.fCurrentLarge ? new Size( 29, 31 ) : new Size( 22, 24 );
			tsb.ToolTipText = this.localizer.DefTooltip;
			tsb.Click += new EventHandler( tsb_Click );
			return tsb;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="ld"></param>
		/// <param name="fLarge"></param>
		/// <returns></returns>
		private ToolStripButtonEx CreateButton( LauncherData ld, bool fLarge )
		{
			ToolStripButtonEx tsb = new ToolStripButtonEx();
			tsb.ImageScaling = ToolStripItemImageScaling.None;
			tsb.AutoSize = false;
			tsb.Size = fLarge ? new Size( 29, 31 ) : new Size( 22, 24 );
			tsb.ToolTipText = MakeTooltipText( ld );
			tsb.LauncherData = ld;
			tsb.SetImage( fLarge );
			tsb.Click += new EventHandler( tsb_Click );
			return tsb;
		}

		/// <summary>
		/// 
		/// </summary>
		private void ReleaseLauncherData()
		{
			foreach( LauncherData ld in this.lstData )
			{
				ld.Dispose();
			}
			this.lstData = null;
		}

		/// <summary>
		/// 
		/// </summary>
		private void RefreshButtons()
		{
			for( int i = 0; i < this.lstBtns.Count; i++ )
			{
				ToolStripButtonEx tsb = this.lstBtns[i];
				if( this.lstData.Count > i )
				{
					// Replace with new LauncherData.
					LauncherData ld = this.lstData[i];
					tsb.LauncherData = ld;
					tsb.ToolTipText = MakeTooltipText( ld );
					tsb.SetImage( this.fCurrentLarge );
				}
				else
				{
					// LauncherData lacks.
					tsb.LauncherData = null;
					tsb.ToolTipText = this.localizer.DefTooltip;
					tsb.Image = this.fCurrentLarge ? Resource.LauncherButton_large : Resource.LauncherButton_small;
				}				
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="ld"></param>
		/// <returns></returns>
		private static string MakeTooltipText( LauncherData ld )
		{
			//return ld.Name + "\n" + ld.ProcessStartInfo.FileName + ( !String.IsNullOrEmpty( ld.ProcessStartInfo.Arguments ) ? "\n" + ld.ProcessStartInfo.Arguments : String.Empty );

			return ld.Name;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void tsb_Click( object sender, EventArgs e )
		{
			if( ( Control.ModifierKeys & Keys.Control ) == Keys.Control )
			{
				this.OnOption();
				return;
			}

			ToolStripButtonEx tsb = (ToolStripButtonEx)sender;

			this.Execute( tsb.LauncherData );

			//if( tsb.LauncherData == null || tsb.LauncherData.ProcessStartInfo == null )
			//{
			//    this.OnOption();
			//    return;
			//}

			//ProcessStartInfo psiOriginal = tsb.LauncherData.ProcessStartInfo;

			//// group, tab
			//if( psiOriginal.FileName != null )
			//{
			//    if( psiOriginal.FileName.StartsWith( PREFIX_GROUP, StringComparison.OrdinalIgnoreCase ) )
			//    {
			//        this.OpenGroup( psiOriginal.FileName.Substring( PREFIX_GROUP.Length ) );
			//        return;
			//    }
			//    else if( psiOriginal.FileName.StartsWith( PREFIX_TAB, StringComparison.OrdinalIgnoreCase ) )
			//    {
			//        this.OpenTab( psiOriginal.FileName.Substring( PREFIX_TAB.Length ) );
			//        return;
			//    }
			//}

			//ProcessStartInfo psi = new ProcessStartInfo( Environment.ExpandEnvironmentVariables( psiOriginal.FileName ) );
			//psi.Arguments = psiOriginal.Arguments;
			//psi.WorkingDirectory = Environment.ExpandEnvironmentVariables( psiOriginal.WorkingDirectory );
			//psi.ErrorDialog = true;
			//psi.ErrorDialogParentHandle = this.pluginServer.ExplorerHandle;

			//AppLauncher.ReplcaceTokens( this.pluginServer, psi );

			//try
			//{
			//    Process.Start( psi );
			//}
			//catch
			//{
			//}
		}

		private void Execute( LauncherData ld )
		{
			if( ld == null || ld.ProcessStartInfo == null )
			{
				this.OnOption();
				return;
			}

			ProcessStartInfo psiOriginal = ld.ProcessStartInfo;

			// group, tab
			if( psiOriginal.FileName != null )
			{
				if( psiOriginal.FileName.StartsWith( PREFIX_GROUP, StringComparison.OrdinalIgnoreCase ) )
				{
					this.OpenGroup( psiOriginal.FileName.Substring( PREFIX_GROUP.Length ) );
					return;
				}
				else if( psiOriginal.FileName.StartsWith( PREFIX_TAB, StringComparison.OrdinalIgnoreCase ) )
				{
					this.OpenTab( psiOriginal.FileName.Substring( PREFIX_TAB.Length ) );
					return;
				}
			}

			ProcessStartInfo psi = new ProcessStartInfo( Environment.ExpandEnvironmentVariables( psiOriginal.FileName ) );
			psi.Arguments = psiOriginal.Arguments;
			psi.WorkingDirectory = Environment.ExpandEnvironmentVariables( psiOriginal.WorkingDirectory );
			psi.ErrorDialog = true;
			psi.ErrorDialogParentHandle = this.pluginServer.ExplorerHandle;

			AppLauncher.ReplcaceTokens( this.pluginServer, psi );

			try
			{
				Process.Start( psi );
			}
			catch
			{
			}
		}

		private void OpenGroup( string strGroup )
		{
			if( !String.IsNullOrEmpty( strGroup ) )
			{
				strGroup = strGroup.Trim( new char[] { '"', '\'' } );

				this.pluginServer.OpenGroup( new string[] { strGroup } );
			}
		}

		private void OpenTab( string path )
		{
			if( !String.IsNullOrEmpty( path ) )
			{
				this.pluginServer.CreateTab( new Address( path ), -1, false, true );
			}
		}


		/// <summary>
		/// Create Launcher Data List.
		/// </summary>
		private int ReadSettings( bool fGetData )
		{
			foreach( LauncherData ld in this.lstData )
			{
				ld.Dispose();
			}
			this.lstData.Clear();

			int c = 0;

			try
			{
				using( RegistryKey rk = Registry.CurrentUser.OpenSubKey( CONSTANTS.REGISTRY_PLUGINSETTINGS + @"\Quizo\LauncherButton" ) )
				{
					if( rk != null )
					{
						foreach( string vn in rk.GetValueNames() )
						{
							string[] strs = Util.ReadRegBinary<string>( vn, rk );
							if( strs != null && strs.Length == 6 )
							{
								c++;
								if( fGetData )
									this.lstData.Add( new LauncherData( strs[0], strs[1], strs[2], strs[3], strs[4], strs[5] ) );
							}
						}
					}
				}
			}
			catch
			{
			}
			return c;
		}

		/// <summary>
		/// 
		/// </summary>
		private void SaveSettings()
		{
			try
			{
				using( RegistryKey rk = Registry.CurrentUser.CreateSubKey( CONSTANTS.REGISTRY_PLUGINSETTINGS + @"\Quizo\LauncherButton" ) )
				{
					if( rk != null )
					{
						foreach( string vn in rk.GetValueNames() )
						{
							rk.DeleteValue( vn );
						}

						for( int i = 0; i < this.lstData.Count; i++ )
						{
							LauncherData ld = this.lstData[i];
							try
							{
								if( !String.IsNullOrEmpty( ld.Name ) && !String.IsNullOrEmpty( ld.ProcessStartInfo.FileName ) )
								{
									string[] strs = new string[] { 
										ld.Name,
										ld.ProcessStartInfo.FileName,
										ld.ProcessStartInfo.Arguments,
										ld.ProcessStartInfo.WorkingDirectory,
										ld.IconPath,
										ld.IconIndex.ToString() };

									Util.WriteRegBinary<string>( strs, i.ToString(), rk );
								}
							}
							catch
							{
							}
						}
					}
				}
			}
			catch
			{
			}

		}

		/// <summary>
		/// 
		/// </summary>
		public static void Uninstall()
		{
			try
			{
				using( RegistryKey rk = Registry.CurrentUser.CreateSubKey( CONSTANTS.REGISTRY_PLUGINSETTINGS + @"\Quizo" ) )
				{
					if( rk != null )
					{
						rk.DeleteSubKeyTree( "LauncherButton" );
					}
				}
			}
			catch
			{
			}

		}

	}

	/// <summary>
	/// 
	/// </summary>
	sealed class LauncherData: IDisposable
	{
		private string name;
		private ProcessStartInfo psi;
		private string iconPath;
		private int iconIndex;
		private bool fHasInstance;
		private Bitmap bmpLarge;
		private Bitmap bmpSmall;


		public LauncherData( string name, string path, string arg, string work, string iconPath, string strIconIndex )
		{
			this.name = name;
			this.psi = new ProcessStartInfo( path );
			psi.Arguments = arg;
			psi.WorkingDirectory = work;

			this.iconPath = iconPath;
			Int32.TryParse( strIconIndex, out this.iconIndex );
		}

		public ProcessStartInfo ProcessStartInfo
		{
			get
			{
				return this.psi;
			}
		}

		public string Name
		{
			get
			{
				return this.name;
			}
		}

		public string IconPath
		{
			get
			{
				return this.iconPath;
			}
		}

		public int IconIndex
		{
			get
			{
				return this.iconIndex;
			}
		}

		public bool Instancialized
		{
			get
			{
				return this.fHasInstance;
			}
			set
			{
				this.fHasInstance = value;
			}
		}

		public Bitmap ImageLarge
		{
			get
			{
				return bmpLarge;
			}
			set
			{
				if( this.bmpLarge != null )
					this.bmpLarge.Dispose();
				bmpLarge = value;
			}
		}

		public Bitmap ImageSmall
		{
			get
			{
				return bmpSmall;
			}
			set
			{
				if( this.bmpSmall != null )
					this.bmpSmall.Dispose();
				bmpSmall = value;
			}
		}

		public void Dispose()
		{
			if( this.bmpLarge != null )
			{
				this.bmpLarge.Dispose();
				this.bmpLarge = null;
			}
			if( this.bmpSmall != null )
			{
				this.bmpSmall.Dispose();
				this.bmpSmall = null;
			}
		}		
	}

	/// <summary>
	/// 
	/// </summary>
	sealed class ToolStripButtonEx : ToolStripButton
	{
		private LauncherData ld;

		public void SetImage( bool fLarge )
		{
			if( this.ld != null )
			{
				try
				{
					Icon icoLarge = null, icoSmall = null;
					if( !String.IsNullOrEmpty( this.ld.IconPath ) )
					{
						string ext = Path.GetExtension( this.ld.IconPath );
						if( SelectIconDialog.EXTS_IMAGE.Contains( ext ) )
						{
							using( Bitmap bmpOriginal = new Bitmap( this.ld.IconPath ) )
							{
								Bitmap bmpLarge = new Bitmap( 24, 24 );
								Bitmap bmpSmall = new Bitmap( 16, 16 );

								using( Graphics g = Graphics.FromImage( bmpLarge ) )
								{
									g.DrawImage( bmpOriginal,
												 new Rectangle( Point.Empty, bmpLarge.Size ),
												 new Rectangle( Point.Empty, bmpOriginal.Size ),
												 GraphicsUnit.Pixel );
								}
								using( Graphics g = Graphics.FromImage( bmpSmall ) )
								{
									g.DrawImage( bmpOriginal,
												 new Rectangle( Point.Empty, bmpSmall.Size ),
												 new Rectangle( Point.Empty, bmpOriginal.Size ),
												 GraphicsUnit.Pixel );
								}
								if( ext == ".bmp" )
								{
									bmpLarge.MakeTransparent( Color.Magenta );
									bmpSmall.MakeTransparent( Color.Magenta );
								}

								this.ld.ImageLarge = bmpLarge;
								this.ld.ImageSmall = bmpSmall;

								this.Image = fLarge ? bmpLarge : bmpSmall;
								return;
							}
						}
						else
						{
							icoLarge = SelectIconDialog.GetIconAt( ld.IconPath, ld.IconIndex, true );
							icoSmall = SelectIconDialog.GetIconAt( ld.IconPath, ld.IconIndex, false );
						}
					}
					else
					{
						icoLarge = Util.GetIcon( this.ld.ProcessStartInfo.FileName, true );
						icoSmall = Util.GetIcon( this.ld.ProcessStartInfo.FileName, false );
					}

					using( icoLarge )
					using( icoSmall )
					{
						if( icoLarge != null && icoSmall != null )
						{
							Bitmap bmpLarge = icoLarge.ToBitmap();
							Bitmap bmpSmall = icoSmall.ToBitmap();
							Bitmap bmpScaled = new Bitmap( 24, 24 );

							if( bmpLarge != null )
							{
								// bmpLarge will be 32x32 or 48x48.
								// scales it down to 24x24.
								using( bmpLarge )
								{
									using( Graphics g = Graphics.FromImage( bmpScaled ) )
									{
										//g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
										g.DrawImage( bmpLarge, new Rectangle( 0, 0, 24, 24 ), new Rectangle( Point.Empty, bmpLarge.Size ), GraphicsUnit.Pixel );
									}
								}
								this.ld.ImageLarge = bmpScaled;
							}
							if( bmpSmall != null )
							{
								this.ld.ImageSmall = bmpSmall;
							}
							this.Image = fLarge ? bmpScaled : bmpSmall;
							return;
						}
					}
				}
				catch
				{
				}
			}

			// if failed, falls back to Resource image.
			this.Image = fLarge ? Resource.LauncherButton_large : Resource.LauncherButton_small;
		}

		public LauncherData LauncherData
		{
			get
			{
				return ld;
			}
			set
			{
				ld = value;
			}
		}
	}

	/// <summary>
	/// 
	/// </summary>
	sealed class Localizer : LocalizedStringProvider
	{
		private string parent;

		public Localizer()
		{
			this.parent = CultureInfo.CurrentCulture.Parent.Name;
		}

		public override string Author
		{
			get
			{
				return "Quizo";
			}
		}

		public override string Name
		{
			get
			{
				if( this.parent == "ja" )
				{
					return "`[{^";
				}
				else
				{
					return "Launcher Buttons";
				}
			}
		}

		public override string Description
		{
			get
			{
				if( this.parent == "ja" )
				{
					return Resource.Description_ja;
				}
				else
				{
					return Resource.Description;
				}
			}
		}

		public string DefTooltip
		{
			get
			{
				if( this.parent == "ja" )
				{
					return "NbNăAvP[Vo^ĂB";
				}
				else
					return "Click and add applications.";
			}
		}

		public string[] DialogResources
		{
			get
			{
				if( this.parent == "ja" )
				{
					return Resource.DialogResources_ja.Split( new char[] { ';' } );
				}
				else
					return null;
			}
		}

		public string ShortcutKeyTemplate
		{
			get
			{
				if( this.parent == "ja" )
				{
					return Resource.ShortcutKeyTemplate_ja;
				}
				else
				{
					return Resource.ShortcutKeyTemplate;
				}
			}
		}


		public override void SetKey( int iKey )
		{
		}
	}
}
