using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace QuizoPlugins
{
	partial class OptionForm : Form
	{
		private const string DEFNAME = "NewItem";
		private const int COLUMN_NAME = 1;
		private const int COLUMN_PATH = 2;
		private const int COLUMN_ARGS = 3;
		private const int COLUMN_WORK = 4;
		private const int COLUMN_ICOP = 5;
		private const int COLUMN_ICOI = 6;

		private List<Icon> lstIcons = new List<Icon>();


		public OptionForm( List<LauncherData> lstLD )
		{
			this.InitializeComponent();

			this.clmIco.DefaultCellStyle.NullValue = null;
			this.clmIco.CellTemplate = new DataGridViewImageCellEx();
			this.dgvLauncher.Rows.Clear();

			this.InitializeDGV( lstLD );
			this.dgvLauncher.CellValueChanged += new DataGridViewCellEventHandler( this.dgvLauncher_CellValueChanged );
		}


		public List<LauncherData> GetData()
		{
			List<LauncherData> lstLDs = new List<LauncherData>();
			List<string> lstNames = new List<string>();

			foreach( DataGridViewRow r in this.dgvLauncher.Rows )
			{
				object oName = r.Cells[COLUMN_NAME].Value;
				object oPath = r.Cells[COLUMN_PATH].Value;
				object oArgs = r.Cells[COLUMN_ARGS].Value;
				object oWork = r.Cells[COLUMN_WORK].Value;
				object oIcoP = r.Cells[COLUMN_ICOP].Value;
				object oIcoI = r.Cells[COLUMN_ICOI].Value;

				if( !r.IsNewRow && oPath != null && oPath.ToString().Length > 0 )
				{
					string name = String.Empty,
						   args = String.Empty,
						   work = String.Empty,
						   icoP = String.Empty;

					if( oName == null )
						name = DEFNAME;
					else
					{
						name = oName.ToString();
						if( name.Length == 0 )
							name = DEFNAME;
					}
					if( name.Length > 64 )
					{
						// 'name' is registry value name.
						name = name.Substring( 0, 64 );
					}

					// make name unique.
					string tmpName = name;
					int iName = 2;
					while( lstNames.Contains( tmpName.ToLower() ) )
					{
						tmpName = name + ( iName++ );
					}
					name = tmpName;

					int iconIndex = 0;
					if( oArgs != null )
						args = oArgs.ToString();
					if( oWork != null )
						work = oWork.ToString();
					if( oIcoP != null )
						icoP = oIcoP.ToString();
					if( oIcoI != null )
					{
						Int32.TryParse( oIcoI.ToString(), out iconIndex );
					}

					lstLDs.Add( new LauncherData( name, oPath.ToString(), args, work, icoP, iconIndex.ToString() ) );
					lstNames.Add( name.ToLower() );
				}
			}

			return lstLDs;
		}

		private void InitializeDGV( List<LauncherData> lstLD )
		{
			this.dgvLauncher.SuspendLayout();

			foreach( LauncherData ld in lstLD )
			{
				DataGridViewRow r = new DataGridViewRow();

				Icon ico = null;
				string iconPath = ld.IconPath;
				int iconIndex = ld.IconIndex;

				if( !String.IsNullOrEmpty( iconPath ) )
				{
					ico = SelectIconDialog.GetIconAt( iconPath, iconIndex, false );
				}
				if( ico == null)
				{
					iconPath = String.Empty;
					iconIndex = 0;
					ico = Util.GetIcon( ld.ProcessStartInfo.FileName, false );
				}
				if( ico != null )
					this.lstIcons.Add( ico );

				r.CreateCells( this.dgvLauncher,
					new object[] { 
						ico,
						ld.Name,
						ld.ProcessStartInfo.FileName,
						ld.ProcessStartInfo.Arguments,
						ld.ProcessStartInfo.WorkingDirectory,
						iconPath,
						iconIndex
					} );


				this.dgvLauncher.Rows.Add( r );
			}

			this.dgvLauncher.ResumeLayout();
		}

		private void MoveRowUpDown( bool fUp )
		{
			if( this.dgvLauncher.SelectedRows.Count == 1 )
			{
				DataGridViewRow row = this.dgvLauncher.SelectedRows[0];
				int index = row.Index;

				if( row.IsNewRow || index < 0 )
					return;

				if( fUp && index > 0 )
				{
					this.dgvLauncher.Rows.Remove( row );
					this.dgvLauncher.Rows.Insert( index - 1, row );

					this.dgvLauncher.ClearSelection();
					row.Selected = true;
				}
				else if( !fUp && index < this.dgvLauncher.Rows.Count - 1 )
				{
					DataGridViewRow rowNext = this.dgvLauncher.Rows[index + 1];
					if( !rowNext.IsNewRow )
					{
						this.dgvLauncher.Rows.Remove( row );
						this.dgvLauncher.Rows.Insert( index + 1, row );

						this.dgvLauncher.ClearSelection();
						row.Selected = true;
					}
				}
			}
		}

		private void ChooseIcon( DataGridViewRow row )
		{
			int index = row.Index;
			if( index > -1 && !row.IsNewRow )
			{
				string strPath		= String.Empty,
					   strIconPath	= String.Empty,
					   strIconIndex = String.Empty;
				int iIConIndex;

				if( row.Cells[COLUMN_PATH].Value != null )
					strPath = row.Cells[COLUMN_PATH].Value.ToString();
				if( row.Cells[COLUMN_ICOP].Value != null )
					strIconPath = row.Cells[COLUMN_ICOP].Value.ToString();
				if( row.Cells[COLUMN_ICOI].Value != null )
					strIconIndex = row.Cells[COLUMN_ICOI].Value.ToString();

				if( !Int32.TryParse( strIconIndex, out iIConIndex ) )
					iIConIndex = -1;

				if( strIconPath.Length == 0 )
					strIconPath = strPath;

				if( strIconPath.Length > 0 )
				{
					using( SelectIconDialog sid = new SelectIconDialog( this, strIconPath, iIConIndex, true ) )
					{
						if( sid.ShowDialog() == DialogResult.OK )
						{
							row.Cells[COLUMN_ICOP].Value = sid.FileName;
							row.Cells[COLUMN_ICOI].Value = sid.Index;

							Icon ico = SelectIconDialog.GetIconAt( sid.FileName, sid.Index, false );
							if( ico != null )
							{
								this.lstIcons.Add( ico );
							}
							row.Cells[0].Value = ico;
						}
					}
				}
			}
		}


		private void dgvLauncher_CellValueChanged( object sender, DataGridViewCellEventArgs e )
		{
			if( e.ColumnIndex == COLUMN_PATH && e.RowIndex > -1 )
			{
				try
				{
					DataGridViewRow row = this.dgvLauncher.Rows[e.RowIndex];
					DataGridViewCell cell = row.Cells[COLUMN_PATH];
					if( cell.Value != null )
					{
						object oIconPath = row.Cells[COLUMN_ICOP].Value;
						if( oIconPath != null && oIconPath.ToString().Length > 0 & File.Exists( oIconPath.ToString() ) )
							return;

						string path = Environment.ExpandEnvironmentVariables( cell.Value.ToString() );
						if( File.Exists( path ) )
						{
							Icon ico = Util.GetIcon( path, false );
							if( ico != null )
								this.lstIcons.Add( ico );

							row.Cells[0].Value = ico;
							return;
						}
					}
					row.Cells[0].Value = null;
				}
				catch
				{
				}
			}
		}

		private void dgvLauncher_CellContentDoubleClick( object sender, DataGridViewCellEventArgs e )
		{
			if( e.ColumnIndex == 0 && -1 < e.RowIndex && e.RowIndex < this.dgvLauncher.Rows.Count )
			{
				this.ChooseIcon( this.dgvLauncher.Rows[e.RowIndex] );
			}
		}

		private void btnUp_Click( object sender, EventArgs e )
		{
			this.MoveRowUpDown( sender == this.btnUp );
		}

		private void tsmiTools_DropDownOpening( object sender, EventArgs e )
		{
			if( this.dgvLauncher.SelectedRows.Count == 1 )
			{
				DataGridViewRow row = this.dgvLauncher.SelectedRows[0];
				int index = row.Index; 

				if( !row.IsNewRow && index > -1 )
				{
					object o = row.Cells[COLUMN_ICOP].Value;
					this.tsmiRestoreIcon.Enabled = ( o != null && o.ToString().Length > 0 );
					this.tsmiChooseIcon.Enabled = true;
					this.tsmiUp.Enabled = index > 0;
					this.tsmiDown.Enabled = index < this.dgvLauncher.Rows.Count - 1;
					return;
				}
			}

			this.tsmiChooseIcon.Enabled = 
			this.tsmiRestoreIcon.Enabled =
			this.tsmiUp.Enabled = 
			this.tsmiDown.Enabled = false;
		}

		private void tsmiInsert_DropDownOpening( object sender, EventArgs e )
		{
			if( this.dgvLauncher.SelectedRows.Count == 1 )
			{
				DataGridViewCell cell = this.dgvLauncher.CurrentCell;

				if( cell.RowIndex == this.dgvLauncher.SelectedRows[0].Index &&
					!this.dgvLauncher.SelectedRows[0].IsNewRow &&
					( cell.ColumnIndex == COLUMN_ARGS || cell.ColumnIndex == COLUMN_WORK ) )
				{
					//arg or work
					this.tsmiSel_Files.Enabled = cell.ColumnIndex == 3;
					this.tsmiSel_Dirs.Enabled =
					this.tsmiSel_All.Enabled =
					this.tsmiSel_Crnt.Enabled =
					this.tsmiSel_CrntOrSel.Enabled = true;
					return;
				}
			}

			this.tsmiSel_Files.Enabled = 
			this.tsmiSel_Dirs.Enabled =
			this.tsmiSel_All.Enabled =
			this.tsmiSel_Crnt.Enabled =
			this.tsmiSel_CrntOrSel.Enabled = false;
		}

		private void tsmiFile_DropDownItemClicked( object sender, ToolStripItemClickedEventArgs e )
		{
			if( e.ClickedItem == this.tsmiOpen )
			{
				this.tsmiFile.DropDown.Close( ToolStripDropDownCloseReason.ItemClicked );

				using( OpenFileDialog ofd = new OpenFileDialog() )
				{
					ofd.RestoreDirectory = false;
					if( DialogResult.OK == ofd.ShowDialog( this ) )
					{
						if( this.dgvLauncher.SelectedRows.Count == 1 )
						{
							DataGridViewRow row = this.dgvLauncher.SelectedRows[0];
							if( !row.IsNewRow )
							{
								row.Cells[COLUMN_PATH].Value = ofd.FileName;
								row.Cells[COLUMN_ICOP].Value = row.Cells[COLUMN_ICOI].Value = null;
								Icon ico = Util.GetIcon( ofd.FileName, false );
								if( ico != null )
									this.lstIcons.Add( ico );
								row.Cells[0].Value = ico;
								return;
							}
						}

						List<LauncherData> lst = new List<LauncherData>();
						LauncherData ld = new LauncherData( 
													DEFNAME,
													ofd.FileName,
													String.Empty,
													Path.GetDirectoryName( ofd.FileName ),
													String.Empty, 
													String.Empty );
						lst.Add( ld );
						this.InitializeDGV( lst );
					}
				}
			}
			else if( e.ClickedItem == this.tsmiClose )
			{
				this.DialogResult = DialogResult.Cancel;
			}
		}

		private void tsmiTools_DropDownItemClicked( object sender, ToolStripItemClickedEventArgs e )
		{
			if( e.ClickedItem == this.tsmiUp || e.ClickedItem == this.tsmiDown )
			{
				this.MoveRowUpDown( e.ClickedItem == this.tsmiUp );
			}
			else if( e.ClickedItem == this.tsmiRestoreIcon )
			{
				if( this.dgvLauncher.SelectedRows.Count == 1 )
				{
					DataGridViewRow row = this.dgvLauncher.SelectedRows[0];
					if( !row.IsNewRow )
					{
						string strPath = String.Empty;
						if( row.Cells[COLUMN_PATH].Value != null )
							strPath = row.Cells[COLUMN_PATH].Value.ToString();

						if( strPath.Length > 0 )
						{
							Icon ico = Util.GetIcon( strPath, false );
							if( ico != null )
								this.lstIcons.Add( ico );

							row.Cells[0].Value = ico;
							row.Cells[COLUMN_ICOP].Value = null;
							row.Cells[COLUMN_ICOI].Value = null;
						}
					}
				}
			}
			else if( e.ClickedItem == this.tsmiChooseIcon )
			{
				if( this.dgvLauncher.SelectedRows.Count == 1 )
				{
					this.ChooseIcon( this.dgvLauncher.SelectedRows[0] );
				}
			}
		}

		private void tsmiInsert_DropDownItemClicked( object sender, ToolStripItemClickedEventArgs e )
		{
			if( this.dgvLauncher.SelectedRows.Count == 1 )
			{
				TextBox textBox = this.dgvLauncher.EditingControl as TextBox;

				string token = String.Empty;
				if( e.ClickedItem == this.tsmiSel_Files )
					token = "%f%";
				else if( e.ClickedItem == this.tsmiSel_Dirs )
					token = "%d%";
				else if( e.ClickedItem == this.tsmiSel_All )
					token = "%s%";
				else if( e.ClickedItem == this.tsmiSel_Crnt )
					token = "%c%";
				else if( e.ClickedItem == this.tsmiSel_CrntOrSel )
					token = "%cd%";

				if( textBox != null )
					textBox.Paste( token );
				else
				{
					DataGridViewCell cell = this.dgvLauncher.CurrentCell;
					if( cell.RowIndex == this.dgvLauncher.SelectedRows[0].Index )
						cell.Value = ( cell.Value == null ? String.Empty : cell.Value.ToString() ) + token;
				}
			}
		}

	}

	sealed class DataGridViewImageCellEx: DataGridViewImageCell
	{
		public DataGridViewImageCellEx()
			: base()
		{
		}
		
		public override Object DefaultNewRowValue
		{
			get
			{
				return null;
			}
		}
	}
}