using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.IO;
using System.Text;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Globalization;

namespace QuizoPlugins
{
	/// <summary>
	/// Represents a common dialog box that allows a user to specify a icon resource path.
	/// </summary>
	partial class SelectIconDialog : CommonDialog
	{
		private SelectIconDialogInnerForm form;

		private static List<ExtractIconData> lstExtractIconData = new List<ExtractIconData>();
		private static int iIndexExtractIcon;
		private static EnumResNameProc enumResNameProc;
		
		public const string SEP_RESOURCE = ", ";
		internal const string EXTS_IMAGE = ".png.bmp.jpg.gif.";

		private Form parentForm;

		///// <summary>
		///// Initializes a new instance of the SelectIconDialog class without specifying a file path. 
		///// </summary>
		//public SelectIconDialog()
		//    : this( null, -1, true )
		//{
		//}

		///// <summary>
		///// Initializes a new instance of the SelectIconDialog class
		///// specifying a file path and icon size to get.
		///// </summary>
		///// <param name="path"></param>
		///// <param name="iconSize"></param>
		//public SelectIconDialog( string path, bool fLargeIcon )
		//    : this( path, -1, fLargeIcon )
		//{
		//}

		public SelectIconDialog( Form parent, string path, int iconIndex, bool fLargeIcon )
		{
			this.parentForm = parent;
			this.form = new SelectIconDialogInnerForm( path, iconIndex, fLargeIcon );
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="disposing"></param>
		protected override void Dispose( bool disposing )
		{
			if( this.form != null )
			{
				this.form.Dispose();
				this.form = null;
			}
			base.Dispose( disposing );
		}

		/// <summary>
		/// Resets all SelectIconDialog properties to their default values. 
		/// </summary>
		public override void Reset()
		{
			this.form.FileName = String.Empty;
			this.form.LargeIcon = true;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="hwndOwner"></param>
		/// <returns></returns>
		protected override bool RunDialog( IntPtr hwndOwner )
		{
			if( this.parentForm != null )
				return this.form.ShowDialog( this.parentForm ) == DialogResult.OK;
			else
				return this.form.ShowDialog() == DialogResult.OK;
		}



		/// <summary>
		/// Extract icons from exe/dll file.
		/// 
		/// cf.	http://jet.breeze.jp/program/tips/icnvbnet.htm
		/// </summary>
		/// <param name="path">full path to the file.</param>
		/// <param name="index">icon index to retrieve. if want all icons to enumerated, specify -1.</param>
		/// <param name="icoSize">IconSize enum value.</param>
		/// <returns>list of icon handles. Caller must release them by DestroyIcon.</returns>
		internal static List<IconPair> GetIconHandles( string path, int index, bool f48pixel )
		{
			const int LOAD_LIBRARY_AS_DATAFILE = 0x00000002;
			const ushort RT_ICON = 3;
			const ushort RT_GROUP_ICON = 14;
			const int LR_DEFAULTCOLOR = 0;

			List<IconPair> lstHICONs = new List<IconPair>();

			IntPtr hInstance = PInvoke.LoadLibraryEx( path, IntPtr.Zero, LOAD_LIBRARY_AS_DATAFILE );
			if( hInstance != IntPtr.Zero )
			{
				try
				{
					// Clear lists for enumerating resources.
					SelectIconDialog.lstExtractIconData.Clear();
					SelectIconDialog.iIndexExtractIcon = 0;

					SelectIconDialog.enumResNameProc = new EnumResNameProc( SelectIconDialog.EnumResNameProcCallBack );
					PInvoke.EnumResourceNames( hInstance, MAKEINTRESOURCE( RT_GROUP_ICON ), SelectIconDialog.enumResNameProc, (IntPtr)index );

					for( int i = 0; i < SelectIconDialog.lstExtractIconData.Count; i++ )
					{
						ExtractIconData eid = SelectIconDialog.lstExtractIconData[i];

						IntPtr hRes;
						if( eid.fID )
							hRes = PInvoke.FindResource( hInstance, (IntPtr)eid.uID, (IntPtr)RT_GROUP_ICON );
						else
							hRes = PInvoke.FindResource( hInstance, eid.strName, (IntPtr)RT_GROUP_ICON );

						if( hRes != IntPtr.Zero )
						{
							IntPtr hMem = PInvoke.LoadResource( hInstance, hRes );
							IntPtr lpv = PInvoke.LockResource( hMem );

							int iSize = f48pixel ? 48 : 32;

							uint idLarge = PInvoke.LookupIconIdFromDirectoryEx( lpv, true, iSize, iSize, LR_DEFAULTCOLOR );
							uint idSmall = PInvoke.LookupIconIdFromDirectoryEx( lpv, true, 16, 16, LR_DEFAULTCOLOR );

							IntPtr hIconLarge = IntPtr.Zero, hIconSmall = IntPtr.Zero;

							if( idLarge != 0 )
							{
								IntPtr hRes2 = PInvoke.FindResource( hInstance, (IntPtr)idLarge, (IntPtr)RT_ICON );
								IntPtr hMem2 = PInvoke.LoadResource( hInstance, hRes2 );
								IntPtr lpv2 = PInvoke.LockResource( hMem2 );

								hIconLarge = PInvoke.CreateIconFromResource( lpv2, PInvoke.SizeofResource( hInstance, hRes2 ), true, 0x00030000 );
							}
							if( idSmall != 0 )
							{
								IntPtr hRes2 = PInvoke.FindResource( hInstance, (IntPtr)idSmall, (IntPtr)RT_ICON );
								IntPtr hMem2 = PInvoke.LoadResource( hInstance, hRes2 );
								IntPtr lpv2 = PInvoke.LockResource( hMem2 );

								hIconSmall = PInvoke.CreateIconFromResource( lpv2, PInvoke.SizeofResource( hInstance, hRes2 ), true, 0x00030000 );
							}

							lstHICONs.Add( new IconPair( hIconLarge, hIconSmall ) );
						}
					}
				}
				finally
				{
					PInvoke.FreeLibrary( hInstance );
				}
				return lstHICONs;
			}
			else
				return null;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="hModule"></param>
		/// <param name="lpszType"></param>
		/// <param name="lpszName"></param>
		/// <param name="lParam"></param>
		/// <returns></returns>
		private static bool EnumResNameProcCallBack( IntPtr hModule, IntPtr lpszType, IntPtr lpszName, IntPtr lParam )
		{
			int indexToGet = (int)lParam;
			bool fTrytoGetAtIndex = indexToGet > -1;

			if( fTrytoGetAtIndex )
			{
				if( indexToGet != SelectIconDialog.iIndexExtractIcon++ )
					return true;
			}

			ExtractIconData eid = new ExtractIconData();
			eid.fID = IS_INTRESOURCE( lpszName );

			if( eid.fID )
			{
				// if IS_INTRESOURCE(lpszName) is TRUE, then lpszName specifies the integer identifier of the given resource. 
				eid.uID = ( (uint)lpszName ) & 0xFFFF;
			}
			else
			{
				// Otherwise, it is a pointer to a null-terminated string.
				eid.strName = Marshal.PtrToStringAuto( lpszName );
			}

			SelectIconDialog.lstExtractIconData.Add( eid );

			return !fTrytoGetAtIndex;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="path"></param>
		/// <param name="fLarge"></param>
		/// <returns></returns>
		private static Icon ExtractIcon( string path, int index, bool fLarge )
		{
			Icon icoRet = null;
			IntPtr[] ppIcons = new IntPtr[1];

			int iExtracted = 0;
			if( fLarge )
				iExtracted = PInvoke.ExtractIconEx( path, index, ppIcons, null, 1 );
			else
				iExtracted = PInvoke.ExtractIconEx( path, index, null, ppIcons, 1 );

			if( ppIcons[0] != IntPtr.Zero )
			{
				using( Icon icoTemp = Icon.FromHandle( ppIcons[0] ) )
				{
					if( icoTemp != null )
					{
						icoRet = (Icon)icoTemp.Clone();
					}
				}
				// Disposing icon constructed using Icon.FromHandle()
				// does not call DestroyIcon API.
				PInvoke.DestroyIcon( ppIcons[0] );
			}
			return icoRet;
		}


		/// <summary>
		/// Extract a icon from exe/dll file.
		/// </summary>
		/// <param name="path">full path to the file.</param>
		/// <param name="index">
		/// icon index to retrieve. this index is from GetIconHandles() not from exe/dll file itself.
		/// It has no meaning outside of this class.
		/// </param>
		/// <param name="fLargeIcon">IconSize enum value.</param>
		/// <returns>Icon instance. Caller must release it by Dispose().</returns>
		public static Icon GetIconAt( string path, int index, bool fLargeIcon )
		{
			if( path == null )
				return null;

			path = Environment.ExpandEnvironmentVariables( path );

			Icon icoRet = null;
			string ext = Path.GetExtension( path ).ToLower();

			if( File.Exists( path ) )
			{
				if( ( ext == ".dll" || ext == ".exe" ) )
				{
					List<IconPair> lstHICON = SelectIconDialog.GetIconHandles( path, index, true );
					if( lstHICON.Count > 0 )
					{
						using( lstHICON[0] )
						{
							IntPtr hIcon = fLargeIcon ? lstHICON[0].hIconLarge : lstHICON[0].hIconSmall;
							if( hIcon != IntPtr.Zero )
							{
								using( Icon icoTemp = Icon.FromHandle( hIcon ) )
								{
									if( icoTemp != null )
									{
										icoRet = (Icon)icoTemp.Clone();
									}
								}

								// Disposing icon constructed using Icon.FromHandle()
								// does not call DestroyIcon API.
							}
						}
					}
				}
				if( EXTS_IMAGE.Contains( ext ) )
				{
					// image file.
					try
					{
						using( Bitmap bmp = new Bitmap( path ) )
						{
							if( ext == ".bmp" )
								bmp.MakeTransparent( Color.Magenta );

						    IntPtr hIcon = bmp.GetHicon();
						    Icon icoTemp = Icon.FromHandle( hIcon );
						    icoRet = (Icon)icoTemp.Clone();

						    PInvoke.DestroyIcon( hIcon );

							//Icon icoTemp = new Icon(
						}
					}
					catch( Exception ex )
					{
						MessageBox.Show( ex.ToString() );
					}
				}
				else
				{
					if( index < 0 )
						index = 0;
					icoRet = SelectIconDialog.ExtractIcon( path, index, fLargeIcon );
				}
			}
			return icoRet;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="path"></param>
		/// <param name="index"></param>
		/// <param name="icoLarge"></param>
		/// <param name="icoSmall"></param>
		/// <returns></returns>
		//public static bool TryGetIconsAt( string path, int index, out Icon icoLarge, out Icon icoSmall )
		//{
		//    icoLarge = icoSmall = null;

		//    if( String.IsNullOrEmpty( path ) || index < 0 )
		//        return false;

		//    string ext = Path.GetExtension( path ).ToLower();

		//    if( ( ext == ".dll" || ext == ".exe" ) && File.Exists( path ) )
		//    {
		//        List<IconPair> lstHICON = SelectIconDialog.GetIconHandles( path, index, false );
		//        if( lstHICON.Count > 0 )
		//        {
		//            using( lstHICON[0] )
		//            {
		//                if( lstHICON[0].hIconLarge != IntPtr.Zero )
		//                {
		//                    using( Icon icoTemp = Icon.FromHandle( lstHICON[0].hIconLarge ) )
		//                    {
		//                        if( icoTemp != null )
		//                        {
		//                            icoLarge = (Icon)icoTemp.Clone();
		//                        }
		//                    }
		//                }
		//                if( lstHICON[0].hIconSmall != IntPtr.Zero )
		//                {
		//                    using( Icon icoTemp = Icon.FromHandle( lstHICON[0].hIconSmall ) )
		//                    {
		//                        if( icoTemp != null )
		//                        {
		//                            icoSmall = (Icon)icoTemp.Clone();
		//                        }
		//                    }
		//                }

		//                return true;

		//                // Disposing icon constructed using Icon.FromHandle()
		//                // does not call DestroyIcon API.
		//            }
		//        }
		//    }
		//    return false;
		//}



		/// <summary>
		/// IS_INTRESOURCE macro.
		/// </summary>
		/// <param name="p"></param>
		/// <returns></returns>
		private static bool IS_INTRESOURCE( IntPtr p )
		{
			//IS_INTRESOURCE(_r) ((((ULONG_PTR)(_r)) >> 16) == 0)
			if( IntPtr.Size == 4 )
			{
				return ( ( (uint)p ) >> 16 ) == 0;
			}
			else
			{
				return ( ( (ulong)p ) >> 16 ) == 0;
			}
		}

		/// <summary>
		/// MAKEINTRESOURCE macro.
		/// </summary>
		/// <param name="wInteger"></param>
		/// <returns></returns>
		private static IntPtr MAKEINTRESOURCE( ushort wInteger )
		{
			//MAKEINTRESOURCEW(i) ((LPWSTR)((ULONG_PTR)((WORD)(i))))
			return new IntPtr( (int)wInteger );
		}



		/// <summary>
		/// Icon size used in Icon List.
		/// </summary>
		public bool LargeIcon
		{
			get
			{
				return this.form.LargeIcon;
			}
			set
			{
				this.form.LargeIcon = value;
			}
		}

		/// <summary>
		/// Gets a string containing the full path and icon index of the icon selected in a icon resource dialog.  
		/// </summary>
		public string ResourceString
		{
			get
			{
				return this.form.ResourceString;
			}
		}

		//[Browsable( false )]
		//public Icon SelectedIcon
		//{
		//    get
		//    {
		//        return this.form.SelectedIcon;
		//    }
		//}

		/// <summary>
		/// Gets or sets a string containing the full path of the file to extract icons.
		/// </summary>
		public string FileName
		{
			get
			{
				return this.form.FileName;
			}
			set
			{
				this.form.FileName = value;
			}
		}

		public int Index
		{
			get
			{
				return this.form.IconIndex;
			}			
		}

		/// <summary>
		/// 
		/// </summary>
		public FormStartPosition StartPosition
		{
			get
			{
				return this.form.StartPosition;
			}
			set
			{
				this.form.StartPosition = value;
			}
		}


		private class ExtractIconData
		{
			/// <summary>
			/// if true, use uID, if false use lpszName
			/// ( in case that resource ID is string )
			/// </summary>
			public bool fID;
			/// <summary>
			/// resource ID (number)
			/// </summary>
			public uint uID;
			/// <summary>
			/// resource ID (string)
			/// </summary>
			public string strName;
		}
	}

	sealed class IconPair : IDisposable
	{
		public IntPtr hIconSmall, hIconLarge;

		public IconPair( IntPtr hIconLarge, IntPtr hIconSmall )
		{
			this.hIconLarge = hIconLarge;
			this.hIconSmall = hIconSmall;
		}

		public void Dispose()
		{
			if( this.hIconSmall != IntPtr.Zero )
			{
				PInvoke.DestroyIcon( this.hIconSmall );
				this.hIconSmall = IntPtr.Zero;
			}
			if( this.hIconLarge != IntPtr.Zero )
			{
				PInvoke.DestroyIcon( this.hIconLarge );
				this.hIconLarge = IntPtr.Zero;
			}
		}
	}




}
