using System;
using System.Collections.Generic;
using System.Drawing;
using System.Globalization;
using System.IO;
using System.Runtime.InteropServices;
using System.Runtime.Remoting.Messaging;
using System.Threading;
using System.Windows.Forms;
using Microsoft.Win32;

using QTPlugin;
using QTPlugin.Interop;

namespace QuizoPlugins
{
	[Plugin( PluginType.Interactive, typeof( Localizer ), 0, Version = "1.1.0.0" )]
	public class SelectEmptyButton : IBarButton
	{
		private IPluginServer pluginServer;
		private IShellBrowser shellBrowser;

		private Localizer localizer;


		#region IPluginClient members

		public void Open( IPluginServer pluginServer, IShellBrowser shellBrowser )
		{
			this.pluginServer = pluginServer;
			this.shellBrowser = shellBrowser;

			this.localizer = new Localizer();
			this.localizer.SetKey( 0 );
		}

		public bool QueryShortcutKeys( out string[] actions )
		{
			actions = new string[] { this.localizer.KeyShortcutStr[0] };
			return true;
		}

		public void Close( EndCode endCode )
		{
		}

		public bool HasOption
		{
			get
			{
				return false;
			}
		}

		public void OnMenuItemClick( MenuType menuType, string menuText, ITab tab )
		{
		}

		public void OnOption()
		{
		}

		public void OnShortcutKeyPressed( int index )
		{
			this.SelectEmpties();
		}

		#endregion


		#region IBarButton members

		public Image GetImage( bool fLarge )
		{
			return fLarge ? Resource.SelectEmptyButton_large : Resource.SelectEmptyButton_small;
		}

		public void InitializeItem()
		{
		}

		public void OnButtonClick()
		{
			this.SelectEmpties();
		}

		public bool ShowTextLabel
		{
			get
			{
				return false;
			}
		}

		public string Text
		{
			get
			{
				return this.localizer.TooltipStr;
			}
		}

		#endregion


		private void SelectEmpties()
		{
			string path = this.pluginServer.SelectedTab.Address.Path;
			if( String.IsNullOrEmpty( path ) )
				return;

			try
			{
				DirectoryInfo di = new DirectoryInfo( path );
				if( !di.Exists )
					return;

				List<Address> lstAddress = new List<Address>();

				foreach( DirectoryInfo diSub in di.GetDirectories() )
				{
					if( FolderIsEmpty( diSub ) )
						lstAddress.Add( new Address( diSub.FullName ) );
				}

				foreach( FileInfo fiSub in di.GetFiles() )
				{
					if( fiSub.Length == 0 )
						lstAddress.Add( new Address( fiSub.FullName ) );
				}

				// event if lstAddress is empty, deselect all.
				this.pluginServer.TrySetSelection( lstAddress.ToArray(), true );
			}
			catch
			{
			}
		}

		private static bool FolderIsEmpty( DirectoryInfo di )
		{
			foreach( FileInfo fiSub in di.GetFiles() )
			{
				if( fiSub.Length > 0 )
					return false;
			}

			foreach( DirectoryInfo diSub in di.GetDirectories() )
			{
				if( !FolderIsEmpty( diSub ) )
					return false;
			}

			return true;
		}
	}

	[Plugin( PluginType.Background, typeof( Localizer ), 1, Version = "1.1.0.0" )]
	public class SelectFileByExtension : IPluginClient
	{
		private IPluginServer pluginServer;
		private Localizer localizer;


		#region IPluginClient Members

		public void Open( IPluginServer pluginServer, IShellBrowser shellBrowser )
		{
			this.pluginServer = pluginServer;

			//if( !pluginServer.TryGetLocalizedStrings( this, 2, out this.ResStr ) )
			//{
			//    this.ResStr = new string[] { StringResources.ButtonNames[9], StringResources.ButtonNames[10] };
			//}

			this.localizer = new Localizer();
			this.localizer.SetKey( 1 );
		}

		public bool QueryShortcutKeys( out string[] actions )
		{
			actions = new string[] { this.localizer.KeyShortcutStr[0], this.localizer.KeyShortcutStr[1] };
			return true;
		}

		public void Close( EndCode endCode )
		{
			this.pluginServer = null;
		}

		public bool HasOption
		{
			get
			{
				return false;
			}
		}

		public void OnMenuItemClick( MenuType menuType, string menuText, ITab tab )
		{
		}

		public void OnOption()
		{
		}

		public void OnShortcutKeyPressed( int index )
		{
			this.SelectFiles( index );
		}

		#endregion


		private void SelectFiles( int index )
		{
			try
			{
				Address[] addresses;
				if( this.pluginServer.TryGetSelection( out addresses ) )
				{
					string ext = null;
					bool fFolder = false;

					foreach( Address ad in addresses )
					{
						if( !String.IsNullOrEmpty( ad.Path ) )
						{
							if( ext == null )
							{
								// first time
								if( Directory.Exists( ad.Path ) )
								{
									fFolder = true;
									ext = "folder";
								}
								else
								{
									ext = Path.GetExtension( ad.Path ).ToLower();
								}
							}
							else if( fFolder )
							{
								if( !Directory.Exists( ad.Path ) )
								{
									this.Deselect();
									return;
								}
							}
							else if( !String.Equals( ext, Path.GetExtension( ad.Path ), StringComparison.OrdinalIgnoreCase ) )
							{
								this.Deselect();
								return;
							}
						}
					}

					if( !String.IsNullOrEmpty( ext ) )
					{
						if( index == 0 )
						{
							string currentPath = this.pluginServer.SelectedTab.Address.Path;
							DirectoryInfo di = new DirectoryInfo( currentPath );
							if( di.Exists )
							{
								List<Address> lstAds = new List<Address>();
								if( fFolder )
								{
									foreach( DirectoryInfo diSub in di.GetDirectories() )
									{
										lstAds.Add( new Address( diSub.FullName ) );
									}
								}
								else
								{
									foreach( FileInfo fi in di.GetFiles( "*" + ext ) )
									{
										lstAds.Add( new Address( fi.FullName ) );
									}
								}

								if( lstAds.Count > 0 )
								{
									this.pluginServer.TrySetSelection( lstAds.ToArray(), true );
								}
							}
						}
						else
						{
							if( !fFolder )
								this.pluginServer.ExecuteCommand( Commands.SetSearchBoxStr, ext );
						}
					}
					else
					{
						this.Deselect();
					}
				}
			}
			catch
			{
			}
		}

		private void Deselect()
		{
			this.pluginServer.TrySetSelection( new Address[] { }, true );
		}

	}

	[Plugin( PluginType.Background, typeof( Localizer ), 2, Version = "1.1.0.0" )]
	public class SelectionInfoToStatusBar : IBarDropButton
	{
		private IPluginServer pluginServer;
		private IShellBrowser shellBrowser;

		private Localizer localizer;

		private delegate void GetInfoInvoker( Address[] addresses );
		private static bool fAborted;
		private volatile bool fThreadRunning;
		private Label ctrl;
		private SelectionData selectionData;

		internal static bool[] currentMode;

		private string[] resStrs, resStrsMenus;
		private static bool IsJa;

		private static bool fEnabledOnNetwork = false;



		#region IPluginClient members

		public void Open( IPluginServer pluginServer, IShellBrowser shellBrowser )
		{
			this.pluginServer = pluginServer;
			this.shellBrowser = shellBrowser;

			this.ctrl = new Label();
			this.ctrl.CreateControl();

			this.localizer = new Localizer();
			this.localizer.SetKey( 2 );

			if( System.Globalization.CultureInfo.CurrentCulture.Parent.Name == "ja" )
			{
				this.resStrs = Resource.selSize_ja.Split( new char[] { ';' } );
				this.resStrsMenus = Resource.sel2status_ja.Split( new char[] { ';' } );
				SelectionInfoToStatusBar.IsJa = true;
			}
			else
			{
				this.resStrs = Resource.selSize.Split( new char[] { ';' } );
				this.resStrsMenus = Resource.sel2status.Split( new char[] { ';' } );
				SelectionInfoToStatusBar.IsJa = false;
			}

			SelectionInfoToStatusBar.currentMode = SelectionInfoToStatusBar.ReadSettings();

			this.pluginServer.NavigationComplete += new PluginEventHandler( pluginServer_NavigationComplete );
			this.pluginServer.SelectionChanged += new PluginEventHandler( pluginServer_SelectionChanged );
		}

		public bool QueryShortcutKeys( out string[] actions )
		{
			actions = new string[] { this.localizer.KeyShortcutStr[0] };		// Abort
			return true;
		}

		public void Close( EndCode endCode )
		{
			// wait until async method returns...
			// may be dangerous...

			SelectionInfoToStatusBar.fAborted = true;

			int timeOut = 0;
			while( this.fThreadRunning )
			{
				Thread.Sleep( 50 );
				if( ++timeOut > 20 )	// 1 sec max.
					break;
			}

			if( endCode == EndCode.WindowClosed )
			{
				this.ctrl.Dispose();
				this.ctrl = null;

				SelectionInfoToStatusBar.SaveSettigns();
			}
		}

		public bool HasOption
		{
			get
			{
				return true;
			}
		}

		public void OnMenuItemClick( MenuType menuType, string menuText, ITab tab )
		{
		}

		public void OnOption()
		{
			using( Form_SelectionInfoSB fs = new Form_SelectionInfoSB() )
			{
				if( DialogResult.OK == fs.ShowDialog() )
				{
					SelectionInfoToStatusBar.currentMode = SelectionInfoToStatusBar.ReadSettings();
				}
			}
		}

		public void OnShortcutKeyPressed( int index )
		{
			bool fRunning = this.fThreadRunning;
			SelectionInfoToStatusBar.fAborted = true;

			if( fRunning )
				this.shellBrowser.SetStatusTextSB( this.resStrs[5] );	// Aborted.
		}

		#endregion


		#region IBarButton members

		public Image GetImage( bool fLarge )
		{
			return fLarge ? Resource.SelectionInfoToStatusBar_large : Resource.SelectionInfoToStatusBar_small;
		}

		public void InitializeItem()
		{
		}

		public void OnButtonClick()
		{
		}

		public bool ShowTextLabel
		{
			get
			{
				return false;
			}
		}

		public string Text
		{
			get
			{
				return this.localizer.Name;
			}
		}

		#endregion


		#region IBarDropButton members

		public bool IsSplitButton
		{
			get
			{
				return false;
			}
		}

		public void OnDropDownOpening( ToolStripDropDownMenu menu )
		{
			if( menu.Items.Count == 0 )
			{
				menu.SuspendLayout();

				menu.ShowImageMargin = false;
				menu.ShowCheckMargin = true;

				for( int i = 0; i < this.resStrsMenus.Length; i++ )
				{
					ToolStripMenuItem tsmi = new ToolStripMenuItem( this.resStrsMenus[i] );
					tsmi.Checked = SelectionInfoToStatusBar.currentMode[i];

					menu.Items.Add( tsmi );
				}
				menu.ResumeLayout();
			}
			else
			{
				for( int i = 0; i < menu.Items.Count; i++ )
				{
					ToolStripMenuItem tsmi = menu.Items[i] as ToolStripMenuItem;
					if( tsmi != null )
					{
						tsmi.Checked = SelectionInfoToStatusBar.currentMode[i];
					}
				}
			}
		}

		public void OnDropDownItemClick( ToolStripItem item, MouseButtons mouseButton )
		{
			SelectionInfoToStatusBar.fAborted = true;

			ToolStrip ts = item.Owner;
			ToolStripMenuItem tsmi = item as ToolStripMenuItem;
			if( ts != null && tsmi != null )
			{
				int index = ts.Items.IndexOf( item );
				if( -1 < index && index < this.resStrsMenus.Length )
				{
					tsmi.Checked = !tsmi.Checked;
					SelectionInfoToStatusBar.currentMode[index] = tsmi.Checked;

					this.StartCalculating();
				}
			}
		}

		#endregion




		/// <summary>
		/// Fired when navigation completed.
		/// Abort getting size.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pluginServer_NavigationComplete( object sender, PluginEventArgs e )
		{
			SelectionInfoToStatusBar.fAborted = true;

			// Current Drive Info
			if( SelectionInfoToStatusBar.currentMode[4] )
			{
				this.SetCurrentDriveInfoText();
			}
		}

		/// <summary>
		/// Fired when selection changed in Folder View.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pluginServer_SelectionChanged( object sender, PluginEventArgs e )
		{
			this.StartCalculating();
		}

		/// <summary>
		/// 
		/// </summary>
		private void StartCalculating()
		{
			SelectionInfoToStatusBar.fAborted = true;
			int timeOut = 0;
			while( this.fThreadRunning )
			{
				Thread.Sleep( 50 );
				if( ++timeOut > 20 )	// 1 sec max.
					break;
			}
			SelectionInfoToStatusBar.fAborted = false;


			// Show mode check ( at least 1 mode is on )
			bool fOn = false;
			for( int i = 0; i < SelectionInfoToStatusBar.currentMode.Length; i++ )
			{
				if( SelectionInfoToStatusBar.currentMode[i] )
				{
					fOn = true;
					break;
				}
			}
			if( !fOn )
				return;


			// StatusBar existance check
			const int FCW_STATUS = 0x0001;
			IntPtr hwndStatusBar;
			if( 0 != this.shellBrowser.GetControlWindow( FCW_STATUS, out hwndStatusBar ) || hwndStatusBar == IntPtr.Zero )
				return;


			Address[] addresses;
			if( this.pluginServer.TryGetSelection( out addresses ) )
			{
				if( addresses == null || addresses.Length == 0 )
				{
					if( SelectionInfoToStatusBar.currentMode[4] )
						this.SetCurrentDriveInfoText();

					return;
				}

				bool fAllDrive = true;
				foreach( Address ad in addresses )
				{
					if( String.IsNullOrEmpty( ad.Path ) || ad.Path.Length != 3 )
					{
						fAllDrive = false;
						break;
					}
				}
				if( fAllDrive )
				{
					this.ShowDriveInfo( addresses );
					return;
				}

				try
				{
					bool fShowStatusGettingSelection = addresses.Length > 1;
					if( !fShowStatusGettingSelection )
					{
						fShowStatusGettingSelection = Directory.Exists( addresses[0].Path );
					}

					if( fShowStatusGettingSelection )
						this.shellBrowser.SetStatusTextSB( this.resStrs[3] );	//Getting selection size...

					this.selectionData = new SelectionData();
					this.fThreadRunning = true;

					GetInfoInvoker gii = new GetInfoInvoker( this.GetInfo );
					gii.BeginInvoke( addresses, new AsyncCallback( this.AsyncComplete ), null );
				}
				catch
				{
					this.shellBrowser.SetStatusTextSB( this.resStrs[4] );	//Failed.
					this.fThreadRunning = false;
				}
			}

		}

		/// <summary>
		/// Callback method, calculates selection size.
		/// This runs in Thread Pool.
		/// </summary>
		/// <param name="addresses"></param>
		private void GetInfo( Address[] addresses )
		{
			long lResult = 0;
			bool fSelectedOne = addresses.Length == 1;

			try
			{
				foreach( Address ad in addresses )
				{
					if( String.IsNullOrEmpty( ad.Path ) ||
						ad.Path.Length == 3 ||
						( !SelectionInfoToStatusBar.fEnabledOnNetwork && ad.Path.StartsWith( @"\\" ) ) )
						continue;

					if( SelectionInfoToStatusBar.fAborted )
					{
						this.selectionData = null;
						return;
					}

					DirectoryInfo di = new DirectoryInfo( ad.Path );
					if( di.Exists )
					{
						this.selectionData.cFolder++;

						// Size
						if( SelectionInfoToStatusBar.currentMode[0] )
						{
							lResult += SelectionInfoToStatusBar.GetDirectorySize( di );
						}

						// mod date
						if( SelectionInfoToStatusBar.currentMode[1] )
						{
							DateTime dt = di.LastWriteTime;

							if( this.selectionData.dtMod < dt )
								this.selectionData.dtMod = dt;

							if( this.selectionData.dtMod_Min > dt )
								this.selectionData.dtMod_Min = dt;
						}

						// crt date
						if( SelectionInfoToStatusBar.currentMode[2] )
						{
							DateTime dt = di.CreationTime;

							if( this.selectionData.dtCrt < dt )
								this.selectionData.dtCrt = dt;

							if( this.selectionData.dtCrt_Min > dt )
								this.selectionData.dtCrt_Min = dt;
						}

						// attributes
						if( fSelectedOne && SelectionInfoToStatusBar.currentMode[3] )
						{
							this.selectionData.attributes = di.Attributes;
						}
					}
					else
					{
						FileInfo fi = new FileInfo( ad.Path );
						if( fi.Exists )
						{
							this.selectionData.cFile++;

							// Size
							if( SelectionInfoToStatusBar.currentMode[0] )
							{
								lResult += fi.Length;
							}

							// mod date
							if( SelectionInfoToStatusBar.currentMode[1] )
							{
								DateTime dt = fi.LastWriteTime;

								if( this.selectionData.dtMod < dt )
									this.selectionData.dtMod = dt;

								if( this.selectionData.dtMod_Min > dt )
									this.selectionData.dtMod_Min = dt;
							}

							// crt date
							if( SelectionInfoToStatusBar.currentMode[2] )
							{
								DateTime dt = fi.CreationTime;

								if( this.selectionData.dtCrt < dt )
									this.selectionData.dtCrt = dt;

								if( this.selectionData.dtCrt_Min > dt )
									this.selectionData.dtCrt_Min = dt;
							}

							// attributes
							if( fSelectedOne && SelectionInfoToStatusBar.currentMode[3] )
							{
								this.selectionData.attributes = fi.Attributes;
							}
						}
					}
				}
			}
			catch
			{
				this.selectionData = null;
				return;
			}

			if( this.selectionData.cFolder == 0 && this.selectionData.cFile < 2 && !SelectionInfoToStatusBar.currentMode[1] && !SelectionInfoToStatusBar.currentMode[2] && !SelectionInfoToStatusBar.currentMode[3] )
			{
				this.selectionData = null;
			}
			else
			{
				this.selectionData.lSize = lResult;
			}
		}

		/// <summary>
		/// Async callback.
		/// </summary>
		/// <param name="ar"></param>
		private void AsyncComplete( IAsyncResult ar )
		{
			AsyncResult asyncResult = (AsyncResult)ar;
			( (GetInfoInvoker)asyncResult.AsyncDelegate ).EndInvoke( ar );

			if( !SelectionInfoToStatusBar.fAborted && this.ctrl != null && !this.ctrl.Disposing && !this.ctrl.IsDisposed && this.ctrl.IsHandleCreated )
			{
				this.ctrl.BeginInvoke( new MethodInvoker( this.ShowResult ) );
			}

			this.fThreadRunning = false;
		}

		/// <summary>
		/// Callback from thread pool.
		/// This runs in Explorer thread.
		/// </summary>
		private void ShowResult()
		{
			if( this.selectionData != null )
			{
				string str = String.Empty;
				if( !SelectionInfoToStatusBar.fAborted )
				{
					bool fSelectedOne = ( this.selectionData.cFolder + this.selectionData.cFile ) == 1;

					// Size
					if( SelectionInfoToStatusBar.currentMode[0] && this.selectionData.lSize > -1 )
					{
						if( this.selectionData.cFolder > 0 )
							str += this.selectionData.cFolder + this.resStrs[0] + MakeS( this.selectionData.cFolder ) + ", ";

						if( this.selectionData.cFile > 0 )
							str += this.selectionData.cFile + this.resStrs[1] + MakeS( this.selectionData.cFile ) + ", ";

						string strSize;
						if( MakeSizeString( this.selectionData.lSize, out strSize ) )
						{
							str += strSize + " (" + this.selectionData.lSize + this.resStrs[2] + MakeS( this.selectionData.lSize ) + ")";
						}
						else
						{
							str += this.selectionData.lSize + this.resStrs[2] + MakeS( this.selectionData.lSize );
						}
					}

					// Mod Date
					if( SelectionInfoToStatusBar.currentMode[1] )
					{
						if( str.Length > 0 )
							str += ",  ";

						str += this.resStrsMenus[1] + ": " + ( fSelectedOne ? String.Empty : this.selectionData.dtMod_Min.ToString() + " - " ) + this.selectionData.dtMod.ToString();
					}

					// Crt Date
					if( SelectionInfoToStatusBar.currentMode[2] )
					{
						if( str.Length > 0 )
							str += ",  ";

						str += this.resStrsMenus[2] + ": " + ( fSelectedOne ? String.Empty : this.selectionData.dtCrt_Min.ToString() + " - " ) + this.selectionData.dtCrt.ToString();
					}

					// Attributes
					if( SelectionInfoToStatusBar.currentMode[3] && fSelectedOne )
					{
						//string strAtt = String.Empty;
						
						////RSHA
						//if( ( this.selectionData.attributes & FileAttributes.ReadOnly ) == FileAttributes.ReadOnly )
						//    strAtt += "R";
						//if( ( this.selectionData.attributes & FileAttributes.System ) == FileAttributes.System )
						//    strAtt += "S";
						//if( ( this.selectionData.attributes & FileAttributes.Hidden ) == FileAttributes.Hidden )
						//    strAtt += "H";

						//if( strAtt.Length > 0 )
						//{
						//    if( str.Length > 0 )
						//        str += ",  ";

						//    str += this.resStrsMenus[3] + ": " + strAtt;
						//}

						if( str.Length > 0 )
							str += ",  ";

						str += this.resStrsMenus[3] + ": " + this.selectionData.attributes.ToString();
					}

					// Current Drive info
					if( SelectionInfoToStatusBar.currentMode[4] )
					{
						try
						{
							string strDrive = this.pluginServer.SelectedTab.Address.Path;
							if( !strDrive.StartsWith( "::" ) )
							{
								DriveInfo di = new DriveInfo( strDrive );
								if( di.IsReady )
								{
									if( str.Length > 0 )
										str += ", ";

									str += MakeSizeString2( di.TotalFreeSpace, this.resStrs[2] ) + "/" + MakeSizeString2( di.TotalSize, this.resStrs[2] );
								}
							}
						}
						catch
						{
						}
					}
				}

				this.shellBrowser.SetStatusTextSB( str );
			}
		}

		/// <summary>
		/// if drives are selected, show drive info.
		/// </summary>
		/// <param name="address"></param>
		private void ShowDriveInfo( Address[] addresses )
		{
			try
			{
				long lTotalFreeSpace = 0;
				long lTotalSize = 0;
				int cDrive = 0;

				DateTime dtMod = DateTime.MinValue;
				DateTime dtCrt = DateTime.MinValue;
				FileAttributes fa = 0;

				bool fSelectedOne = addresses.Length == 1;

				foreach( Address ad in addresses )
				{
					if( !String.Equals( ad.Path, @"a:\", StringComparison.OrdinalIgnoreCase ) &&
						!String.Equals( ad.Path, @"b:\", StringComparison.OrdinalIgnoreCase ) &&
						( SelectionInfoToStatusBar.fEnabledOnNetwork || !ad.Path.StartsWith( @"\\" ) ) )
					{
						DriveInfo dr = new DriveInfo( ad.Path );
						if( dr.IsReady )
						{
							cDrive++;

							if( SelectionInfoToStatusBar.currentMode[0] )
							{
								lTotalFreeSpace += dr.TotalFreeSpace;
								lTotalSize += dr.TotalSize;
							}

							if( fSelectedOne )
							{
								if( SelectionInfoToStatusBar.currentMode[1] || SelectionInfoToStatusBar.currentMode[2] || SelectionInfoToStatusBar.currentMode[3] )
								{
									DirectoryInfo di = new DirectoryInfo( ad.Path );
									if( di.Exists )
									{
										// mod date
										if( SelectionInfoToStatusBar.currentMode[1] )
										{
											dtMod = di.LastWriteTime;
										}

										// crt date
										if( SelectionInfoToStatusBar.currentMode[2] )
										{
											dtCrt = di.CreationTime;
										}

										// attributes
										if( SelectionInfoToStatusBar.currentMode[3] )
										{
											fa = di.Attributes;
										}
									}
								}
							}
						}
					}
				}

				if( cDrive > 1 )
				{
					string strTotalFreeSize, strTotalSize;
					MakeSizeString( lTotalFreeSpace, out strTotalFreeSize );
					MakeSizeString( lTotalSize, out strTotalSize );

					if( strTotalFreeSize != null && strTotalSize != null )
					{
						this.shellBrowser.SetStatusTextSB( this.resStrs[6] + strTotalFreeSize + " " + this.resStrs[7] + strTotalSize );
					}
				}
				else if( cDrive == 1 )
				{
					string strInfo = String.Empty;

					// mod date
					if( SelectionInfoToStatusBar.currentMode[1] )
					{
						strInfo += this.resStrsMenus[1] + ": " + dtMod.ToString();
					}

					// crt date
					if( SelectionInfoToStatusBar.currentMode[2] )
					{
						if( strInfo.Length > 0 )
							strInfo += ",  ";

						strInfo += this.resStrsMenus[2] + ": " + dtCrt.ToString();
					}

					// attributes
					if( SelectionInfoToStatusBar.currentMode[3] )
					{
						if( strInfo.Length > 0 )
							strInfo += ",  ";

						strInfo += this.resStrsMenus[3] + ": " + fa.ToString();
					}

					if( strInfo.Length > 0 )
					{
						this.shellBrowser.SetStatusTextSB( strInfo );
					}
				}
			}
			catch
			{
			}
		}

		/// <summary>
		/// 
		/// </summary>
		private void SetCurrentDriveInfoText()
		{
			const int FCW_STATUS = 0x0001;
			const uint WM_USER = 0x0400;
			const uint SB_GETTEXTLENGTHW = ( WM_USER + 12 );
			const uint SB_GETTEXTW = ( WM_USER + 13 );

			try
			{
				IntPtr p;
				if( 0 == this.shellBrowser.SendControlMsg( FCW_STATUS, SB_GETTEXTLENGTHW, IntPtr.Zero, IntPtr.Zero, out p ) )
				{
					int len = (int)p;
					len &= 0xFFFF;

					IntPtr pText = Marshal.AllocHGlobal( len * 2 );
					if( pText != IntPtr.Zero )
					{
						if( 0 == this.shellBrowser.SendControlMsg( FCW_STATUS, SB_GETTEXTW, IntPtr.Zero, pText, out p ) )
						{
							try
							{
								string s = Marshal.PtrToStringUni( pText );
								string path = this.pluginServer.SelectedTab.Address.Path;
								if( !path.StartsWith( "::" ) )
								{
									DriveInfo di = new DriveInfo( path );
									if( di.IsReady )
									{
										this.shellBrowser.SetStatusTextSB( s + "              " + MakeSizeString2( di.TotalFreeSpace, this.resStrs[2] ) + "/" + MakeSizeString2( di.TotalSize, this.resStrs[2] ) );
									}
								}
							}
							catch
							{
							}
						}
						Marshal.FreeHGlobal( pText );
					}
				}
			}
			catch
			{
			}
		}


		/// <summary>
		/// Get selection size recursively.
		/// </summary>
		/// <param name="di"></param>
		/// <returns></returns>
		private static long GetDirectorySize( DirectoryInfo di )
		{
			if( SelectionInfoToStatusBar.fAborted )
				return -1;

			long lSize = 0;

			try
			{
				foreach( DirectoryInfo diSub in di.GetDirectories() )
				{
					if( SelectionInfoToStatusBar.fAborted )
						return -1;
					try
					{
						lSize += GetDirectorySize( diSub );
					}
					catch
					{
						// access error, etc.
					}
				}

				foreach( FileInfo fiSub in di.GetFiles() )
				{
					if( SelectionInfoToStatusBar.fAborted )
						return -1;

					try
					{
						lSize += fiSub.Length;
					}
					catch
					{
						// access error, etc.
					}
				}
			}
			catch
			{
				// enumeration error, etc.
				lSize = -1;
			}

			return lSize;
		}


		private static string MakeS( long c )
		{
			return ( !IsJa && c > 1 ) ? "s" : String.Empty;
		}

		private static bool MakeSizeString( long lSize, out string str )
		{
			if( lSize >= 1073741824 )
			{
				str = Math.Round( lSize / 1073741824d, 2, MidpointRounding.AwayFromZero ) + " GB";
				return true;
			}
			if( lSize >= 1048576 )
			{
				str = Math.Round( lSize / 1048576d, 2, MidpointRounding.AwayFromZero ) + " MB";
				return true;
			}
			if( lSize >= 1024 )
			{
				str = Math.Round( lSize / 1024d, 2, MidpointRounding.AwayFromZero ) + " KB";
				return true;
			}

			str = null;
			return false;
		}

		/// <summary>
		/// this.resStrs[2]
		/// </summary>
		/// <param name="lSize"></param>
		/// <param name="strByte"></param>
		/// <returns></returns>
		private static string MakeSizeString2( long lSize, string strByte )
		{
			if( lSize >= 1073741824 )
			{
				return Math.Round( lSize / 1073741824d, 2, MidpointRounding.AwayFromZero ) + " GB";
			}
			if( lSize >= 1048576 )
			{
				return Math.Round( lSize / 1048576d, 2, MidpointRounding.AwayFromZero ) + " MB";
			}
			if( lSize >= 1024 )
			{
				return Math.Round( lSize / 1024d, 2, MidpointRounding.AwayFromZero ) + " KB";
			}
			else
			{
				return lSize + " " + strByte + MakeS( lSize );
			}
		}


		class SelectionData
		{
			public int cFolder;
			public int cFile;
			public long lSize;

			public DateTime dtMod;
			public DateTime dtMod_Min = DateTime.MaxValue;
			public DateTime dtCrt;
			public DateTime dtCrt_Min = DateTime.MaxValue;

			public FileAttributes attributes;
		}


		#region Settings

		internal static void SaveSettigns()
		{
			byte b = 0;
			if( currentMode[0] )
				b |= 0x80;
			if( currentMode[1] )
				b |= 0x40;
			if( currentMode[2] )
				b |= 0x20;
			if( currentMode[3] )
				b |= 0x10;
			if( currentMode[4] )
				b |= 0x08;

			using( RegistryKey rk = Registry.CurrentUser.CreateSubKey( CONSTANTS.REGISTRY_PLUGINSETTINGS + @"\Quizo\SelectionInfoToStatusBar" ) )
			{
				rk.SetValue( "Config", new byte[] { b, 0, 0, 0 } );
			}
		}

		internal static bool[] ReadSettings()
		{
			bool[] modes = new bool[5];

			using( RegistryKey rk = Registry.CurrentUser.OpenSubKey( CONSTANTS.REGISTRY_PLUGINSETTINGS + @"\Quizo\SelectionInfoToStatusBar" ) )
			{
				if( rk != null )
				{
					byte[] bytes = (byte[])rk.GetValue( "Config" );
					if( bytes != null && bytes.Length > 0 )
					{
						modes[0] = ( bytes[0] & 0x80 ) != 0;
						modes[1] = ( bytes[0] & 0x40 ) != 0;
						modes[2] = ( bytes[0] & 0x20 ) != 0;
						modes[3] = ( bytes[0] & 0x10 ) != 0;
						modes[4] = ( bytes[0] & 0x08 ) != 0;
					}
				}
				else
				{
					modes[0] = true;
				}
			}

			return modes;
		}

		public static void Uninstall()
		{
			try
			{
				using( RegistryKey rk = Registry.CurrentUser.OpenSubKey( CONSTANTS.REGISTRY_PLUGINSETTINGS + @"\Quizo", true ) )
				{
					if( rk != null )
					{
						rk.DeleteSubKeyTree( "SelectionInfoToStatusBar" );
					}
				}
			}
			catch
			{
			}
		}


		#endregion
	}



	sealed class StringResources
	{
		public static string[] Names, Descriptions, KeyShortcuts, Tooltips;

		static StringResources()
		{
			char[] SEPCHAR = new char[] { ';' };

			if( CultureInfo.CurrentCulture.Parent.Name == "ja" )
			{
				Names		 = Resource.name_ja.Split( SEPCHAR );
				Descriptions = Resource.dcpt_ja.Split( SEPCHAR );
				KeyShortcuts = Resource.keys_ja.Split( SEPCHAR );
				Tooltips	 = Resource.tooltip_ja.Split( SEPCHAR );
			}
			else
			{
				Names		 = Resource.name.Split( SEPCHAR );
				Descriptions = Resource.dcpt.Split( SEPCHAR );
				KeyShortcuts = Resource.keys.Split( SEPCHAR );
				Tooltips	 = Resource.tooltip.Split( SEPCHAR );
			}
		}
	}

	sealed class Localizer : LocalizedStringProvider
	{
		private string parent;
		private int iKey;

		public Localizer()
		{
			this.parent = CultureInfo.CurrentCulture.Parent.Name;
		}

		public override string Author
		{
			get
			{
				return "Quizo";
			}
		}

		public override string Description
		{
			get
			{
				return StringResources.Descriptions[this.iKey];
			}
		}

		public override string Name
		{
			get
			{
				return StringResources.Names[this.iKey];
			}
		}

		public override void SetKey( int iKey )
		{
			this.iKey = iKey;
		}

		public string[] KeyShortcutStr
		{
			get
			{
				switch( this.iKey )
				{
					case 0 :
						return new string[] { StringResources.KeyShortcuts[0] };

					case 1:
						return new string[] { StringResources.KeyShortcuts[1], StringResources.KeyShortcuts[2] };

					case 2:
						return new string[] { StringResources.KeyShortcuts[3] };

					default:
						return new string[0];
				}
			}
		}

		public string TooltipStr
		{
			get
			{
				return StringResources.Tooltips[this.iKey];
			}
		}
	}
}
